@@ -96,6 +96,12 @@ class TemplatedString(object):
             return ''
         return unicode(self.val)
 
+    def __nonzero__(self):
+        if self.val:
+            return True
+        else:
+            return False
+
     def __call__(self):
         renderer = self.obj.form.renderer
         name = '%s_%s'%(self.obj.type,self.attr_name)
@@ -536,11 +542,6 @@ class Form(object):
 
     The Form type is the container for all the information a form needs to
     render and validate data.
-
-    :var name: The name of the form, used to namespace classes and ids
-    :var defaults: Property to allow getting and setting of defaults
-    :var error: Used to indicate a non field specific error on the form. None
-                if no error
     """    
 
     renderer = _default_renderer
@@ -572,8 +573,6 @@ class Form(object):
 
         :arg renderer: Something that returns a form serialization when called
         :type renderer: callable
-
-
         """
         # allow a single schema items to be used on a form
         if not isinstance(structure, schemaish.Structure):
@@ -612,7 +611,7 @@ class Form(object):
             return self._element_name
         if self._name is not None:
             return self._name
-        return 'formish'
+        return 'form'
 
     def _name_set(self, name):
         """ Set the form name """ 
