@@ -35,7 +35,7 @@ def build_request(formname, data):
 #
 #  Simple Types
 
-def String():
+def String(request):
     """
     A simple form with a single string field
     """
@@ -44,7 +44,8 @@ def String():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_String(self, sel):#{{{
+def functest_String(self):#{{{
+    sel = self.selenium
     sel.open("/String")
 
     sel.click("form-action-submit")
@@ -68,23 +69,23 @@ def functest_String(self, sel):#{{{
 
 def unittest_String(self):#{{{
     # Test no data
-    f = String()
+    f = String(None)
     self.assertIdHasValue(f, 'form-myStringField', '')
     # Test None data
-    f = String()
+    f = String(None)
     testdata = {'myStringField': None}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myStringField', '')
     self.assertRoundTrip(f, testdata)
     # Test sample data
-    f = String()
+    f = String(None)
     testdata = {'myStringField': '8'}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myStringField', '8')
     self.assertRoundTrip(f, testdata)#}}}
 
 
-def Integer():
+def Integer(request):
     """
     A simple form with a single integer field
     """
@@ -93,7 +94,8 @@ def Integer():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_Integer(self,sel):#{{{
+def functest_Integer(self):#{{{
+    sel = self.selenium
     sel.open("/Integer")
 
     sel.click("form-action-submit")
@@ -123,22 +125,22 @@ def functest_Integer(self,sel):#{{{
 
 def unittest_Integer(self):#{{{
     # Test no data
-    f = Integer()
+    f = Integer(None)
     self.assertIdHasValue(f, 'form-myIntegerField', '')
     # Test None data
-    f = Integer()
+    f = Integer(None)
     testdata = {'myIntegerField': None}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myIntegerField', '')
     self.assertRoundTrip(f, testdata)
     # Test sample data
-    f = Integer()
+    f = Integer(None)
     testdata = {'myIntegerField': 8}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myIntegerField', '8')
     self.assertRoundTrip(f, testdata)#}}}
 
-def Date():
+def Date(request):
     """
     A simple form with a single integer field
     """
@@ -147,7 +149,8 @@ def Date():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_Date(self, sel):#{{{
+def functest_Date(self):#{{{
+    sel = self.selenium
 
     sel.open("/Date")
 
@@ -176,7 +179,7 @@ def functest_Date(self, sel):#{{{
 
     return#}}}
 
-def Float():
+def Float(request):
     """
     A simple form with a single integer field
     """
@@ -185,7 +188,8 @@ def Float():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_Float(self, sel):#{{{
+def functest_Float(self):#{{{
+    sel = self.selenium
     sel.open("/Float")
 
     sel.click("form-action-submit")
@@ -215,7 +219,7 @@ def functest_Float(self, sel):#{{{
 
 
 
-def Boolean():
+def Boolean(request):
     """
     A simple form with a single boolean field
     """
@@ -224,7 +228,8 @@ def Boolean():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_Boolean(self, sel):#{{{
+def functest_Boolean(self):#{{{
+    sel = self.selenium
     sel.open("/Boolean")
 
     sel.click("form-action-submit")
@@ -259,7 +264,7 @@ def functest_Boolean(self, sel):#{{{
     return#}}}
 
 
-def Decimal():
+def Decimal(request):
     """
     A simple form with a single decimal field
     """
@@ -269,7 +274,8 @@ def Decimal():
     return form
 
 
-def functest_Decimal(self, sel):#{{{
+def functest_Decimal(self):#{{{
+    sel = self.selenium
     sel.open("/Decimal")
 
     sel.click("form-action-submit")
@@ -297,7 +303,7 @@ def functest_Decimal(self, sel):#{{{
 
     return#}}}
 
-def RequiredStringAndCheckbox():
+def RequiredStringAndCheckbox(request):
     """
     Testing that a checkbox is working properly when another required field is missing
     """
@@ -308,7 +314,8 @@ def RequiredStringAndCheckbox():
     form['myBoolean'].widget=formish.Checkbox()
     return form
 
-def functest_RequiredStringAndCheckbox(self, sel):#{{{
+def functest_RequiredStringAndCheckbox(self):#{{{
+    sel = self.selenium
     sel.open("/RequiredStringAndCheckbox")
 
     sel.click("form-action-submit")
@@ -331,7 +338,7 @@ def functest_RequiredStringAndCheckbox(self, sel):#{{{
 
     return#}}}
 
-def File():
+def File(request):
     """
     A simple form with a single integer field
     """
@@ -341,7 +348,8 @@ def File():
     form['myFile'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
     return form
 
-def functest_File(self, sel):#{{{
+def functest_File(self):#{{{
+    sel = self.selenium
     sel.open("/File")
 
     sel.click("form-action-submit")
@@ -364,7 +372,7 @@ def functest_File(self, sel):#{{{
 #   String Widgets
 #
 
-def Input():
+def Input(request):
     """
     Simple input field with a strip parameter
     """
@@ -375,7 +383,7 @@ def Input():
     form['inputStrip'].widget = formish.Input(strip=True)
     return form
 
-def Hidden():
+def Hidden(request):
     """
     Hidden Field with a visible friend.. 
     """
@@ -387,7 +395,7 @@ def Hidden():
     form['Hidden'].widget = formish.Hidden()
     return form
 
-def Password():
+def Password(request):
     """
     Using a password html element
     """
@@ -398,7 +406,7 @@ def Password():
     form['Password'].widget = formish.Password()
     return form
 
-def CheckedPassword():
+def CheckedPassword(request):
     """
     Simple input field with a strip parameter
     """
@@ -409,7 +417,7 @@ def CheckedPassword():
     form['CheckedPassword'].widget = formish.CheckedPassword()
     return form
 
-def TextAreaSimple():
+def TextAreaSimple(request):
     """
     Simple text area
     """
@@ -420,7 +428,7 @@ def TextAreaSimple():
     form['textArea'].widget = formish.TextArea()
     return form
 
-def TextAreaColsAndRows():
+def TextAreaColsAndRows(request):
     """
     Passing cols and rows to a text area
     """
@@ -431,7 +439,7 @@ def TextAreaColsAndRows():
     form['textAreaCustom'].widget = formish.TextArea(cols=20,rows=4)
     return form
 
-def TextAreaStrip():
+def TextAreaStrip(request):
     """
     Stripping text area input
     """
@@ -446,7 +454,7 @@ def TextAreaStrip():
 #
 #   Validation
 
-def Required():
+def Required(request):
     """
     Required Fields
     """
@@ -456,7 +464,7 @@ def Required():
     form = formish.Form(schema, 'form')
     return form
 
-def MinLength():
+def MinLength(request):
     """
     Minimum Length fields - this one is min length four chars
     """
@@ -466,7 +474,7 @@ def MinLength():
     form = formish.Form(schema, 'form')
     return form
 
-def MaxLength():
+def MaxLength(request):
     """
     Maximum Length fields - this one is max length eight chars
     """
@@ -476,7 +484,7 @@ def MaxLength():
     form = formish.Form(schema, 'form')
     return form
 
-def MinMaxLength():
+def MinMaxLength(request):
     """
     Minimum and maximum length
     """
@@ -486,7 +494,7 @@ def MinMaxLength():
     form = formish.Form(schema, 'form')
     return form
 
-def MinLengthCheckboxMultiChoice():
+def MinLengthCheckboxMultiChoice(request):
     """
     A checkbox multi choice with minimum length 3 (NOT WORKING)
     """
@@ -500,7 +508,7 @@ def MinLengthCheckboxMultiChoice():
     form['multiChoice'].widget = formish.CheckboxMultiChoice(options)
     return form
 
-def MinRange():
+def MinRange(request):
     """
     Minimum Value fields - this one is min value of 4
     """
@@ -510,7 +518,7 @@ def MinRange():
     form = formish.Form(schema, 'form')
     return form
 
-def MaxRange():
+def MaxRange(request):
     """
     Maximum value for a field - this one is max of 8
     """
@@ -520,7 +528,7 @@ def MaxRange():
     form = formish.Form(schema, 'form')
     return form
 
-def MinMaxRange():
+def MinMaxRange(request):
     """
     Minimum and maximum value
     """
@@ -530,7 +538,7 @@ def MinMaxRange():
     form = formish.Form(schema, 'form')
     return form
 
-def PlainText():
+def PlainText(request):
     """
     Plain Text (alphanum only)
     """
@@ -540,7 +548,7 @@ def PlainText():
     form = formish.Form(schema, 'form')
     return form
 
-def OneOf():
+def OneOf(request):
     """
     One of a set of
     """
@@ -551,7 +559,7 @@ def OneOf():
     form = formish.Form(schema, 'form')
     return form
 
-def All():
+def All(request):
     """
     Required and Plain Text (alphanum only plus _ and -)
     """
@@ -564,10 +572,22 @@ def All():
     form = formish.Form(schema, 'form')
     return form
 
-
-def ValidationOnSequenceItem():
+def ReCAPTCHA(request):
     """
-    A structure witin a sequence, should be enhanced with javascript
+    ReCAPTCHA widget
+    """
+    schema = schemaish.Structure()
+    schema.add('recaptcha', schemaish.Boolean())
+
+    form = formish.Form(schema, 'form')
+    publickey = '6LcSqgQAAAAAAA1A6MJZXGpY35ZsdvwxvsEq0KQD'
+    privatekey = '6LcSqgQAAAAAAGn0bfmasP0pGhKgF7ugn72Hi2va'
+    form['recaptcha'].widget = xformish.ReCAPTCHA(publickey, privatekey, request.environ)
+    return form
+
+def ValidationOnSequenceItem(request):
+    """
+    Validation on a sequence item.
     """
     substructure = schemaish.Structure()
     substructure.add( 'a', schemaish.String(validator=validatish.Required()) )
@@ -579,7 +599,22 @@ def ValidationOnSequenceItem():
     form = formish.Form(schema, 'form')
     return form
 
-def RequiredStringAndFile():
+def ValidationOnSequence(request):
+    """
+    Validation on a collection (sequence).
+    """
+    substructure = schemaish.Structure()
+    substructure.add( 'a', schemaish.String() )
+    substructure.add( 'b', schemaish.Integer() )
+
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( substructure, validator=validatish.Length(min=2)))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+
+def RequiredStringAndFile(request):
     """
     A required string and a file field configured for image upload.
     """
@@ -590,7 +625,8 @@ def RequiredStringAndFile():
     form['myFileField'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb(),show_image_preview=True,originalurl='/images/nouploadyet.png')
     return form
 
-def functest_RequiredStringAndFile(self, sel):#{{{
+def functest_RequiredStringAndFile(self):#{{{
+    sel = self.selenium
     sel.open("/RequiredStringAndFile")
 
     sel.click("form-action-submit")
@@ -638,21 +674,23 @@ def functest_RequiredStringAndFile(self, sel):#{{{
     actualfilepath = 'cache/%s'%filesrc
     assert os.path.exists(actualfilepath)
     stdout = subprocess.Popen([IDENTIFY, actualfilepath], stdout=subprocess.PIPE).communicate()[0]
-    assert stdout.endswith('photo.jpg JPEG 300x300 300x300+0+0 DirectClass 8-bit 4.36133kb \n')
+    assert 'photo.jpg JPEG 300x300' in stdout
     
     actualfilepath = '%s-100x100'%actualfilepath
     assert os.path.exists(actualfilepath)
     stdout = subprocess.Popen([IDENTIFY, actualfilepath], stdout=subprocess.PIPE).communicate()[0]
-    assert stdout.endswith('photo.jpg-100x100 JPEG 100x100 100x100+0+0 DirectClass 8-bit 1.70703kb \n')
+    assert 'photo.jpg-100x100 JPEG 100x100' in stdout
 
           
     return#}}}
 
+
+
 ########################
 #
 #   Checkbox
 
-def Checkbox():
+def Checkbox(request):
     """
     Simple Boolean Checkbox
     """
@@ -669,7 +707,7 @@ def Checkbox():
 #   Select Widgets
 #
 
-def SelectChoice():
+def SelectChoice(request):
     """
     A basic select choice
     """
@@ -682,7 +720,7 @@ def SelectChoice():
     return form
 
 def unittest_SelectChoice(self):#{{{
-    f = SelectChoice()
+    f = SelectChoice(None)
     testdata = {'mySelect': 3}
     f.defaults = testdata
     request = build_request('form',testdata)
@@ -691,7 +729,7 @@ def unittest_SelectChoice(self):#{{{
     #}}}
 
 
-def SelectChoiceNoneOption():
+def SelectChoiceNoneOption(request):
     """
     Setting a None Option on the select choice element
     """
@@ -703,13 +741,13 @@ def SelectChoiceNoneOption():
     form['mySelect'].widget = formish.SelectChoice(options,none_option=(None, '--select--'))
     return form
 
-def SelectChoiceCallableOptions():
+def SelectChoiceCallableOptions(request):
     """
     Passing in a callable list of options
     """
     schema = schemaish.Structure()
     schema.add('mySelect', schemaish.Integer())
-    def _():
+    def _(request):
         options = [(1,'a'),(2,'b'),(3,'c')]
         for option in options:
             yield option
@@ -718,7 +756,7 @@ def SelectChoiceCallableOptions():
     form['mySelect'].widget = formish.SelectChoice(_)
     return form
 
-def SelectWithOtherChoice():
+def SelectWithOtherChoice(request):
     """
     A basic select choice
     """
@@ -730,7 +768,8 @@ def SelectWithOtherChoice():
     form['mySelect'].widget = formish.SelectWithOtherChoice(options)
     return form
 
-def functest_SelectWithOtherChoice(self, sel):#{{{
+def functest_SelectWithOtherChoice(self):#{{{
+    sel = self.selenium
     sel.open("/SelectWithOtherChoice")
 
     try: self.assertEqual("", sel.get_value("form-mySelect"))
@@ -760,7 +799,7 @@ def functest_SelectWithOtherChoice(self, sel):#{{{
 
 def unittest_SelectWithOtherChoice(self):#{{{
     # Test no data
-    f = SelectWithOtherChoice()
+    f = SelectWithOtherChoice(None)
     testdata = {'mySelect': 4}
     f.defaults = testdata
     reqdata = {'mySelect.other':'4','mySelect.select':'...'}
@@ -768,7 +807,7 @@ def unittest_SelectWithOtherChoice(self):#{{{
     data = f.validate(request)
     assert data == testdata
 
-    f = SelectWithOtherChoice()
+    f = SelectWithOtherChoice(None)
     testdata = {'mySelect': 2}
     f.defaults = testdata
     reqdata = {'mySelect.other':'','mySelect.select':'2'}
@@ -777,7 +816,7 @@ def unittest_SelectWithOtherChoice(self):#{{{
     assert data == testdata
     return #}}}
 
-def RadioChoice():
+def RadioChoice(request):
     """
     A basic select choice
     """
@@ -790,7 +829,7 @@ def RadioChoice():
     return form
 
 
-def RadioChoiceNoneOption():
+def RadioChoiceNoneOption(request):
     """
     Setting a None Option on the select choice element
     """
@@ -802,13 +841,13 @@ def RadioChoiceNoneOption():
     form['myRadio'].widget = formish.RadioChoice(options,none_option=(None, '--select--'))
     return form
 
-def RadioChoiceCallableOptions():
+def RadioChoiceCallableOptions(request):
     """
     Passing in a callable list of options
     """
     schema = schemaish.Structure()
     schema.add('myRadio', schemaish.Integer())
-    def _():
+    def _(request):
         options = [(1,'a'),(2,'b'),(3,'c')]
         for option in options:
             yield option
@@ -822,7 +861,7 @@ def RadioChoiceCallableOptions():
 #   Multi Select Widgets
 #
 
-def CheckboxMultiChoice():
+def CheckboxMultiChoice(request):
     """
     A checkbox representing a set of values
     """
@@ -835,7 +874,7 @@ def CheckboxMultiChoice():
     return form
 
 
-def CheckboxMultiChoiceTree():
+def CheckboxMultiChoiceTree(request):
     """
     A checkbox representing a set of values
     """
@@ -855,7 +894,7 @@ def CheckboxMultiChoiceTree():
 #   Structures
 #
 
-def SimpleStructure():
+def SimpleStructure(request):
     """
     A simple structure
     """
@@ -869,7 +908,7 @@ def SimpleStructure():
     form = formish.Form(schema, 'form')
     return form
   
-def UploadStructure():
+def UploadStructure(request):
     """
     A structure with a file upload
     """
@@ -893,7 +932,7 @@ def UploadStructure():
 #
 
 
-def SequenceOfSimpleStructures():
+def SequenceOfSimpleStructures(request):
     """
     A structure witin a sequence, should be enhanced with javascript
     """
@@ -907,7 +946,7 @@ def SequenceOfSimpleStructures():
     form = formish.Form(schema, 'form')
     return form
   
-def SequenceOfUploadStructures():
+def SequenceOfUploadStructures(request):
     """
     A structure witin a sequence, should be enhanced with javascript
     """
@@ -923,7 +962,8 @@ def SequenceOfUploadStructures():
     form['myList.*.a'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
     return form
 
-def functest_SequenceOfUploadStructures(self, sel):#{{{
+def functest_SequenceOfUploadStructures(self):#{{{
+    sel = self.selenium
 
     sel.open("/SequenceOfUploadStructures")
     sel.wait_for_page_to_load("30000")
@@ -941,7 +981,7 @@ def functest_SequenceOfUploadStructures(self, sel):#{{{
 
     return#}}}
 
-def SequenceOfStructuresWithSelects():
+def SequenceOfStructuresWithSelects(request):
     """
     A sequence including selects
     """
@@ -960,7 +1000,8 @@ def SequenceOfStructuresWithSelects():
     form.defaults = {'myList': [{'a':'foo','b':'b'}]}
     return form
 
-def functest_SequenceOfStructuresWithSelects(self, sel):#{{{
+def functest_SequenceOfStructuresWithSelects(self):#{{{
+    sel = self.selenium
     sel.open("/SequenceOfStructuresWithSelects")
 
     try: self.assertEqual("foo", sel.get_value("form-myList-0-a"))
@@ -978,7 +1019,7 @@ def functest_SequenceOfStructuresWithSelects(self, sel):#{{{
 
     return#}}}
 
-def SequenceOfDateParts():
+def SequenceOfDateParts(request):
     """
     A test of a sequence of more complicated structures
     """
@@ -992,7 +1033,10 @@ def SequenceOfDateParts():
     form['birthdays.*.date'].widget = formish.DateParts()
     return form
 
-def functest_SequenceOfDateParts(self, sel):#{{{
+
+
+def functest_SequenceOfDateParts(self):#{{{
+    sel = self.selenium
     sel.open("/SequenceOfDateParts")
 
     sel.click_at("css=#form-birthdays-field > a", "")
@@ -1001,8 +1045,18 @@ def functest_SequenceOfDateParts(self, sel):#{{{
 
     return#}}}
 
+def SequenceOfSequencesAsTextArea(request):
+    """
+    A simple text area but representing a csv style data structure
+    """
+    schema = schemaish.Structure()
+    schema.add('table', schemaish.Sequence(schemaish.Tuple( (schemaish.String(), schemaish.Integer(), schemaish.Date()) )))
+    form = formish.Form(schema)
+    form['table'].widget = formish.TextArea()
+    return form
+    
 
-def SequenceOfStructures():
+def SequenceOfStructures(request):
     """
     A test of a sequence of more complicated structures
     """
@@ -1023,7 +1077,7 @@ def SequenceOfStructures():
     return form
 
 
-def GranularFormLayout():
+def GranularFormLayout(request):
     """
     A simple demonstration of partial rendering of parts of forms.
     """
@@ -1038,7 +1092,7 @@ def GranularFormLayout():
     return form
 
 
-def template_GranularFormLayout():
+def template_GranularFormLayout(request):
     """
 
 ${form()|n}
@@ -1131,7 +1185,7 @@ ${form['firstName'].description()}
     """
 
     
-def CustomisedFormLayout():
+def CustomisedFormLayout(request):
     """
     A custom form
     """
@@ -1146,7 +1200,7 @@ def CustomisedFormLayout():
     return form
 
 
-def template_CustomisedFormLayout():
+def template_CustomisedFormLayout(request):
     """
 ${form.header()|n}
 ${form.metadata()|n}
