@@ -84,7 +84,7 @@ class Root(resource.Resource):
     
     def resource_child(self, request, segments):
         if segments[0] == 'filehandler':
-            return fileresource.FileResource(fileaccessor=FileAccessor(),segments=segments[1:]), ()
+            return fileresource.FileResource(fileaccessor=FileAccessor()), segments[1:]
         return FormResource(segments[0]), segments[1:]
 
 
@@ -111,27 +111,24 @@ class FormResource(resource.Resource):
         else:
             tests = '<a href="?show_tests=True">Click here to see tests</a>'
         if form is None:
-            form = self.form_getter()
+            form = self.form_getter(request)
             form.renderer = request.environ['restish.templating.renderer']
         return {'title': self.title, 'description': self.description,
                 'form': form, 'data': pformat(data),'rawdata': data,
                 'template': extract_function.extract_docstring('template_%s'%self.id),
                 'template_highlighted': extract_function.extract_docstring_highlighted('template_%s'%self.id),
-                'definition': extract_function.extract(self.id),
+                'definition': extract_function.extract(self.id).replace('6LcSqgQAAAAAAGn0bfmasP0pGhKgF7ugn72Hi2va','6LcSqgQA......................ugn72Hi2va'),
                 'tests': tests}
     
     @resource.POST()
     def POST(self, request):
-        form = self.form_getter()
+        form = self.form_getter(request)
         form.renderer = request.environ['restish.templating.renderer']
         try:
             data = form.validate(request)
         except formish.FormError, e:
             return self.render_form(request, form=form)
         else:
-            print request.environ
-            print request.headers
-            print '[',request.body,']'
             if 'myFileField' in data:
                 f = data['myFileField']
                 filedata = f.file.read()
