@@ -149,10 +149,15 @@ class TestErrorRendering(base.TestCase):
         ERROR_TEXT = '!!!WOOP!!!WOOP!!!WOOP!!!'
         form = formish.Form(schema, errors={attr: ERROR_TEXT})
         html = form()
-        element = BeautifulSoup(html).find(id='formish-%s-field'%(attr.replace('.', '-'),))
-        error = element.find('span', {'class': 'error'})
-        assert error
-        assert error.string == ERROR_TEXT
+        element = BeautifulSoup(html).find(id='form-%s-field'%(attr.replace('.', '-'),))
+        div_error = element.find('div',{'class': 'error'})
+        span_error = element.find('span',{'class': 'error'})
+        assert div_error or span_error
+        if div_error:
+            assert div_error.string == ERROR_TEXT 
+        if span_error:
+            assert span_error.string == ERROR_TEXT 
+
 
 
 if __name__ == '__main__':
