@@ -42,7 +42,7 @@ class Widget(object):
         return [string_data]
 
 
-    def pre_parse_request(self, schema_type, request_data):
+    def pre_parse_request(self, schema_type, request_data, full_request_data):
         """
         Prior to convert being run, we have a chance to munge the data. This is
         only used by file upload at the moment
@@ -168,6 +168,7 @@ class SequenceDefault(Widget):
         self.max = k.get('max')
         self.min = k.get('min')
         self.addremove = k.get('addremove', True)
+        self.sortable = k.get('sortable', True)
         self.converttostring = False
 
     def pre_render(self, schema_type, data):
@@ -326,7 +327,7 @@ class FileUpload(Widget):
             self.default = ''
         return {'name': [self.default], 'default':[self.default]}
     
-    def pre_parse_request(self, schema_type, data):
+    def pre_parse_request(self, schema_type, data, full_request_data):
         """
         File uploads are wierd; in out case this means assymetric. We store the
         file in a temporary location and just store an identifier in the field.
@@ -375,6 +376,11 @@ class SelectChoice(Widget):
     none_option = ('', '- choose -')
 
     def __init__(self, options, none_option=UNSET, css_class=None):
+        """
+        :arg options: either a list of values ``[value,]`` where value is used for the label or a list of tuples of the form ``[(value, label),]``
+        :arg none_option: a tuple of ``(value, label)`` to use as the unselected option
+        :arg css_class: a css class to apply to the field
+        """
         Widget.__init__(self)
         self.css_class = css_class
         self.options = _normalise_options(options)
